/*
	$Id: avi.c,v 1.1 2007/02/12 04:31:57 jogi Exp $
*/

#include "common.h"
#include "quakedef.h"
#include <vfw.h>
#include "mvd_cam.h"
#pragma comment( lib, "vfw32.lib" )
extern int numgltextures;

#define Q_ROUND_POWER2(in, out) {						\
	_mathlib_temp_int1 = in;							\
	for (out = 1; out < _mathlib_temp_int1; out <<= 1)	\
	;												\
}

GLfloat	avi_plane_points[4][2];

GLfloat	avi_plane_points[4][2] = {{1.0,0.0},{1.0,1.0},{0.0,0.0},{0.0,1.0}};


int	avi_texnum;
int tempwidth,tempheight;
double	avi_time;
int		avi_current_frame;

HBITMAP hBitmap;	
AVISTREAMINFO		psi;						// Pointer To A Structure Containing Stream Info
PAVISTREAM			pavi;						// Handle To An Open Stream
PGETFRAME			pgf;						// Pointer To A GetFrame Object
BITMAPINFOHEADER	bmih;						// Header Information For DrawDibDraw Decoding
long				lastframe;					// Last Frame Of The Stream
int					width;						// Video Width
int					height;						// Video Height
char				*pdata;						// Pointer To Texture Data
int					mpf;	
unsigned char* data = 0;	
HDC hde ;
HDRAWDIB hdd;
int avi_playing;

void OpenAVI(LPCSTR szFile)						// Opens An AVI File (szFile)
{
	LPBITMAPINFOHEADER lpbi;					// Holds The Bitmap Header Information
	// Opens The AVI Stream
	if (AVIStreamOpenFromFile(&pavi, szFile, streamtypeVIDEO, 0, OF_READ, NULL) !=0)
	{
		// An Error Occurred Opening The Stream
			Com_Printf("AVI system: error - failed to open file\n");
	}
	AVIStreamInfo(pavi, &psi, sizeof(psi));				// Reads Information About The Stream Into psi
	width=psi.rcFrame.right-psi.rcFrame.left;			// Width Is Right Side Of Frame Minus Left
	height=psi.rcFrame.bottom-psi.rcFrame.top;			// Height Is Bottom Of Frame Minus Top

	lastframe=AVIStreamLength(pavi);				// The Last Frame Of The Stream

	mpf=AVIStreamSampleToTime(pavi,lastframe)/lastframe;		// Calculate Rough Milliseconds Per Frame
	bmih.biSize		= sizeof (BITMAPINFOHEADER);		// Size Of The BitmapInfoHeader
	bmih.biPlanes		= 1;					// Bitplanes
	bmih.biBitCount		= 24;					// Bits Format We Want (24 Bit, 3 Bytes)
	bmih.biWidth		= 256;					// Width We Want (256 Pixels)
	bmih.biHeight		= 256;					// Height We Want (256 Pixels)
	bmih.biCompression	= BI_RGB;				// Requested Mode = RGB
	
	//hBitmap = CreateDIBSection (hde, (BITMAPINFO*)(&bmih), DIB_RGB_COLORS, (void**)(&data), NULL, NULL);
	//SelectObject (hde, hBitmap);					// Select hBitmap Into Our Device Context (hdc)

	pgf=AVIStreamGetFrameOpen(pavi, AVIGETFRAMEF_BESTDISPLAYFMT);				// Create The PGETFRAME Using Our Request Mode
	if (pgf==NULL)
	{
		Com_Printf("AVI system: Codec couldnt be found\n");
	}
	
	lpbi = (LPBITMAPINFOHEADER)AVIStreamGetFrame(pgf, 1);	// Grab Data From The AVI Stream
	pdata=(char *)lpbi+lpbi->biSize+lpbi->biClrUsed * sizeof(RGBQUAD);
	avi_texnum = GL_LoadTexturePixels(pdata,"avi_tex",width,height,0);
	
	Com_Printf("%i %i %i %i %i %i %i\n",mpf,lastframe,width,height,avi_texnum,tempwidth,tempheight);
	

}

void GrabAVIFrame(int frame)						// Grabs A Frame From The Stream
{
	LPBITMAPINFOHEADER lpbi;					// Holds The Bitmap Header Information
	lpbi = (LPBITMAPINFOHEADER)AVIStreamGetFrame(pgf, frame);	// Grab Data From The AVI Stream
	pdata=(char *)lpbi+lpbi->biSize+lpbi->biClrUsed * sizeof(RGBQUAD);	// Pointer To Data Returned By AVIStreamGetFrame
										// (Skip The Header Info To Get To The Data)
	// Convert Data To Requested Bitmap Format
	//DrawDibDraw (hdd, hde, 0, 0, tempwidth, tempheight, lpbi, pdata, 0, 0, width, height, 0);
	GL_Bind(avi_texnum);
	//Com_Printf("%s\n",pdata);
	glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, width, height, GL_BGRA, GL_UNSIGNED_BYTE, pdata);

}
void avi_load_f(void){
	
	OpenAVI(Cmd_Argv(1));
	
}

void avi_close_f(void){
	//DeleteObject(hBitmap);						// Delete The Device Dependant Bitmap Object
	AVIStreamGetFrameClose(pgf);					// Deallocates The GetFrame Resources
	AVIStreamRelease(pavi);						// Release The Stream
	AVIFileExit();							// Release The File

}

void avi_frame_f(void){
	//Com_Printf("%i\n",atoi(Cmd_Argv(1)));
	GrabAVIFrame(atoi(Cmd_Argv(1)));
}

void avi_play_f(void){
	avi_current_frame = -1;
	avi_playing = 1;
	avi_time = cls.realtime - 1;
}

void avi_stop_f(void){
	avi_playing = 0;
}

void avi_frame(void){
	if (avi_playing)
		if (avi_current_frame < lastframe-1){
			if (avi_time + 1/mpf < cls.demotime ){
				//Com_Printf("%i %f %f\n",avi_current_frame,avi_time + 1/mpf,cls.realtime);
				avi_time = cls.demotime;
				avi_current_frame++;
				GrabAVIFrame(avi_current_frame);
			}
		}else{
			avi_playing = 0;
		}

}


void avi_draw_plane(void){
	extern tele_plane_t tele_cams[1];
	glMap2f(GL_MAP2_VERTEX_3, 0.0, 1.0, 3, 2, 0.0, 1.0, 6, 2,tele_cams[0].setpoints[0]);

	glMap2f(GL_MAP2_TEXTURE_COORD_2, 0.0, 1.0, 2, 2, 0.0, 1.0, tele_cams[0].test, tele_cams[0].test1, avi_plane_points[0]);
	glEnable(GL_MAP2_TEXTURE_COORD_2);
	GL_Bind(avi_texnum);
	glEnable(GL_MAP2_VERTEX_3);
	glEnable(GL_AUTO_NORMAL);
	glEnable(GL_NORMALIZE);
	glMapGrid2f(40, 0.0, 1.0, 40, 0.0, 1.0);
	glEvalMesh2(GL_FILL, 0, 40, 0, 40);

}


void avi_init(void){
	AVIFileInit();							// Opens The AVIFile Library
	Cmd_AddCommand("avi_load",avi_load_f);
	Cmd_AddCommand("avi_close",avi_close_f);
	Cmd_AddCommand("avi_frame",avi_frame_f);
	Cmd_AddCommand("avi_play",avi_play_f);
	Cmd_AddCommand("avi_stop",avi_stop_f);
	//avi_texnum = numgltextures++;
	//hde = CreateCompatibleDC(0);

	
}
