/*

*/

#include "quakedef.h"

int particles_count = 0;
void PS_Rotate_Vector (vec3_t dst,vec3_t org,float degree_x, float degree_y);
#define Cprintv(v,n) Com_Printf("%s - %f %f %f\n",n,v[0],v[1],v[2]);
#define Cprintt(a,c)			\
do {							\
	if (!c){					\
		Com_Printf("%s\n",a);	\
	}							\
	c = true;					\
} while (0);

#define DRAW_PARTICLE_BILLBOARD(_ptex, _p, _coord)			\


//	glScalef(_p->size, _p->size, _p->size);					\
//	if (_p->rotspeed)										\
//		glRotatef(_p->rotangle, vpn[0], vpn[1], vpn[2]);	
#define V_Distance(a,b) sqrt(pow((a[0]- b[0]),2) + pow((a[1]- b[1]),2) + pow((a[2]- b[2]),2));
static float sint[7] = {0.000000, 0.781832, 0.974928, 0.433884, -0.433884, -0.974928, -0.781832};
static float cost[7] = {1.000000, 0.623490, -0.222521, -0.900969, -0.900969, -0.222521, 0.623490};

typedef enum { pt_bounce = 28 , pt_time = 15, pt_hit = 5, pt_origin = 8, pt_gravity = 6, pt_speed = 17, pt_direction = 1, pt_ival = 12, pt_color = 31, pt_sound = 9,pt_type = 2,pt_textures = 4, pt_size = 27, pt_light = 24};


typedef struct ps_g_point_s {
	vec3_t origin;
	float	gravity;
	float	distance;
} ps_g_point_t;

ps_g_point_t gravity_point[10];

void mvd_vectoangles(vec3_t vec, vec3_t ang);

vec3_t	geradeaus;
vec3_t	kommwerher,kuckmerhin;
int	daswinkel = 0;
int	*daswinkel_pointer = &daswinkel;

int	dasanderewinkel = 0;
int	*dasanderewinkel_pointer = &dasanderewinkel;


int unique_light_id = 10000;

qbool	flipme ;

typedef struct p_info_s {
	qbool	active;
	qbool	dies_by;
	int		blends_by;
	float	value;
	float	lmax;
	float	lmin;
	char	*spawns;
	float	modifier;
} p_info_t;

typedef struct p_light_s {
	int				unique_id;
	int				color;
	int				type;
	float			radius;
	float			time;
	qbool			custom;
	float			custom_color[3];
	
} p_light_t;

typedef struct p_color_s {
	int		blends_by;
	float	color[4];
	float	color_start[4];
	float	color_end[4];
} p_color_t;

typedef struct p_random_s {
	int		type;
	float	min;
	float	max;
	float	value1;
	float	value2;
} p_random_t;


typedef struct ps_child_s {
	char			*identifier;					// name
	int				hash;							// like the cvars :>
	int				type;							// billboard etc etc
	float			starttime;						// time we spawned
	p_info_t		time;
	p_info_t		interval;
	p_info_t		bounce;
	p_light_t		light;
	p_info_t		speed;
	p_info_t		impact;
	p_color_t		color;
	p_random_t		random_direction;
	p_random_t		random_origin;

	//qbool			dies_by_impact;					// we do expire when whe hit something
	qbool			g_affected;						// are we affected by gravity
	//qbool			done;							// er ?
	float			g_modifier;						// gravity modifier
	char			*on_death_spawn;				// list of particles spawned when particle dies
	//char			*on_impact_spawn;				// list of particles spawned when particle impact

	vec3_t			origin;							// origin
	vec3_t			old_origin[9];
	vec3_t			direction;						// direction

	

	sfx_t			*sound;
	mpic_t			texture;
	qbool			sound_played;
	float			size;
	float			growth;
	//int				blend_over_time;				// 1 = start at 0 end at 255

} ps_child_t;

typedef struct ps_plist_s {
	struct ps_plist_s	*next;
	struct ps_child_s	*particle;
} ps_plist_t;

ps_plist_t	*loaded_particles_list = NULL;
ps_plist_t	*draw_particles_list = NULL;

typedef struct ps_parent_s {
	float	time;
	char	*identifier;
	vec3_t	origin;
	vec3_t	direction;
	char	*plist;

} ps_parent_t;

typedef struct ps_parent_list_s {
	struct ps_parent_list_s	*next;
	struct ps_parent_s	*parent;
} ps_parent_list_t;

ps_parent_list_t	*ps_parent_list = NULL ;


void PS_Draw_Particles(void){

	int j,k;
	ps_child_t	*p;
	vec3_t v, billboard[4];
	ps_plist_t *my_list;
	my_list = draw_particles_list;
	
	VectorAdd(vup, vright, billboard[2]);
	VectorSubtract(vright, vup, billboard[3]);
	VectorNegate(billboard[2], billboard[0]);
	VectorNegate(billboard[3], billboard[1]);

	glDepthMask(GL_FALSE);
	glEnable(GL_BLEND);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glShadeModel(GL_SMOOTH);


	glDisable(GL_TEXTURE_2D);

	glPointSize(5.0);
	glColor3f(1,1,0);
	while (my_list){
		
		p = my_list->particle;
		if (p->type == 0){
			glDisable(GL_TEXTURE_2D);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glBegin(GL_POINTS);
			//Com_Printf("%s\n",my_list->particle->identifier);
			glColor4fv(p->color.color);
			glVertex3fv(p->origin);
		}
		
		if (p->type == 1){
			glDisable(GL_TEXTURE_2D);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glBegin(GL_TRIANGLE_FAN);
			glColor4fv(p->color.color);
			glVertex3fv(p->origin);
			//glColor4ub(p->color[0] >> 1, p->color[1] >> 1, p->color[2] >> 1, 0);

			for (j = 7; j >= 0; j--) {
				for (k = 0; k < 3; k++)
					v[k] = p->origin[k] - p->direction[k] / 8 + vright[k] * cost[j % 7] * 1 + vup[k] * sint[j % 7] * 1;
				glVertex3fv(v);
			}
		}

		if (p->type == 2){
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_COLOR);
			if (!p->texture.texnum){
				Com_Printf("Test !\n");
				continue;
			}

			glEnable(GL_TEXTURE_2D);
			//VULT PARTICLES - I gather this speeds it up, but I haven't really checked
			GL_Bind(p->texture.texnum);
			
			glPushMatrix();
			glTranslatef(p->origin[0], p->origin[1], p->origin[2]);
			glScalef(p->size, p->size, p->size);
			glColor4fv(p->color.color);
			glBegin(GL_QUADS);
			glTexCoord2f(1.0f, 1.0f); glVertex3fv(billboard[0]);
			glTexCoord2f(0.0f, 1.0f); glVertex3fv(billboard[1]);
			glTexCoord2f(0.0f, 0.0f); glVertex3fv(billboard[2]);
			glTexCoord2f(1.0f, 0.0f); glVertex3fv(billboard[3]);
	
		}


		glEnd();
		if (p->type == 2)
			glPopMatrix();
		glFlush();
		if (my_list->next)
			my_list=my_list->next;
		else
			break;
	}
	
	glDepthMask(GL_TRUE);
	glDisable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_TEXTURE_2D);

	glPointSize(5.0);
	glDisable(GL_TEXTURE_2D);
	glColor3f(0,0,1);
	glBegin(GL_POINTS);
	glVertex3fv(gravity_point[0].origin);
	glVertex3fv(gravity_point[1].origin);
	glVertex3fv(gravity_point[2].origin);
	glEnd();
	glBegin(GL_LINES);
	glVertex3fv(gravity_point[0].origin);
	glVertex3fv(gravity_point[1].origin);
	glEnd();
	glBegin(GL_LINES);
	glVertex3fv(gravity_point[0].origin);
	glVertex3fv(gravity_point[2].origin);
	glEnd();
	glFlush();
	glEnable(GL_TEXTURE_2D);


}

static qbool TraceLineN (vec3_t start, vec3_t end, vec3_t impact, vec3_t normal) {
	trace_t trace;

	memset (&trace, 0, sizeof(trace));
	trace.fraction = 1;
	if (PM_RecursiveHullCheck (cl.worldmodel->hulls, 0, 0, 1, start, end, &trace))
		return false;
	VectorCopy (trace.endpos, impact);
	if (normal)
		VectorCopy (trace.plane.normal, normal);

	return true;
}

#define TruePointContents(p) PM_HullPointContents(&cl.worldmodel->hulls[0], 0, p)


void CL_NewDlight_RGB (int key, vec3_t origin, float radius, float time, int type, int bubble, qbool color, float r, float g, float b);
void PS_Spawn_Particles_From_Particle(char *identifier,ps_child_t *parent);

void PS_Check_Particles(void){
	ps_plist_t *last;
	ps_plist_t *current;
	ps_plist_t *temp;
	ps_child_t *p;
	qbool dead,hit,test;
	char *p_id,*buf;
	vec3_t	oldorg,stop,normal,gp_accel;
	int i;
	float bounce, grav,distance;


	flipme = !flipme;

	//if (flipme)
//		return;

	current = draw_particles_list;
	grav = movevars.gravity / 800.0;




	while (current && current->particle){
		dead = false;
		hit = false;
		test = true;
		p = current->particle;

		// time.modifer wich is (currenttime - starttime)/time.value
		p->time.modifier = (r_refdef2.time - p->starttime) / p->time.value;


		// check if p is dead :D
		if (!p->sound_played){
			S_StartSound (cl.playernum+1, -1, p->sound, p->origin, 0.5, 1);
			//S_StaticSound_OneTime(,p->on_bounce_spawn,0.5,0.5);
			p->sound_played = true;
		}
		if (r_refdef2.time > (p->starttime + p->time.value) && p->time.dies_by)
			dead = true;
		
		if (dead)
			Cprintt("i died from time\n",test);
/*
		if (p->speed < p->dies_by_speed_limit && p->dies_by_speed == 1 && !p->time.dies_by)
			dead = true;

		if (dead)
			Cprintt("i died from speed\n",test);

		if (p->speed > p->dies_by_speed_limit && p->dies_by_speed == 2 && !p->time.dies_by)
			dead = true;
		if (dead)
			Cprintt("i died from speed\n",test);
*/
		if (p->bounce.dies_by && p->bounce.lmax < 0)
			dead = true;
		if (dead)
			Cprintt("i died from bouncelimit\n",test);

		
		if (p->interval.value != 0){
			if (p->interval.modifier  < r_refdef2.time){
				p->interval.modifier = r_refdef2.time + p->interval.value;
				buf = Q_strdup(p->interval.spawns);
				p_id = strtok(buf, " ");
				while ( p_id != NULL){
					PS_Spawn_Particles_From_Particle(p_id,p);
					p_id = strtok(NULL, " ");
				}
				Q_free(buf);
			}
		}


		// we blend thats just transparency
		if (p->time.blends_by == 1){
			p->color.color[3] = ((r_refdef2.time -p->starttime)/p->time.dies_by);
		}
		if (p->time.blends_by == 2){
			p->color.color[3] = 1 - ((r_refdef2.time -p->starttime)/p->time.dies_by);
		}

		// this is color over time
		if (p->color.blends_by == 1){
			for (i=0;i<3;i++){
				p->color.color[i] = p->color.color_start[i] + (p->color.color_end[i] - p->color.color_start[i]) * p->time.modifier;
				//Com_Printf("%f\n",p->color.color[i]);
			}
		}


		// apply acceleration
		VectorScale(p->direction, 1 + p->speed.modifier * cls.frametime, p->direction)
		// apply gravity
		p->direction[2] -= p->g_modifier * grav * cls.frametime;

		// gravity points
		for (i=0;i<2;i++){
			distance = V_Distance(p->origin,gravity_point[i].origin)
			if (distance <= gravity_point[i].distance){
				VectorSubtract(gravity_point[i].origin, p->origin, gp_accel);
				//AngleVectors(gp_accel,forward,right,up);
				VectorNormalize(gp_accel);
				VectorMA(p->direction,  (distance/gravity_point[i].distance) * gravity_point[i].gravity , gp_accel, p->direction);
			}
		}


		// move it and check if we hit smth
		VectorCopy(p->origin, oldorg);
		VectorMA(p->origin, cls.frametime, p->direction, p->origin);
		if (-2 == TruePointContents (p->origin)) {
			if (TraceLineN(oldorg, p->origin, stop, normal)){
				VectorCopy(stop, p->origin);
				hit = true;
			}
		}


		VectorCopy(oldorg,p->old_origin[0]);

		if (p->light.unique_id){
			CL_NewDlight_RGB(p->light.unique_id,p->origin, p->light.radius, p->light.time, p->light.type, 0,p->light.custom,p->light.custom_color[0],p->light.custom_color[1],p->light.custom_color[2]);

			}

		if (p->bounce.active && hit && VectorCompare(p->origin,oldorg))
			dead = true;
		if (dead)
			Cprintt("i died cuz i dont bounce\n",test);

		if (p->bounce.active && hit && !VectorCompare(p->origin,oldorg)){
			bounce = -p->bounce.value * DotProduct(p->direction, normal);
			VectorMA(p->direction, bounce, normal, p->direction);
			p->bounce.lmax--;
			if (p->bounce.spawns != NULL){
				buf = Q_strdup(p->bounce.spawns);
				p_id = strtok(buf, " ");
				while ( p_id != NULL){
					PS_Spawn_Particles_From_Particle(p_id,p);
					p_id = strtok(NULL, " ");
				}
				Q_free(buf);
			}
		}
		

		if (p->impact.dies_by && hit){
			if (p->impact.spawns != NULL){
				
				buf = Q_strdup(p->impact.spawns);
				p_id = strtok(buf, " ");
				while ( p_id != NULL){
					PS_Spawn_Particles_From_Particle(p_id,p);
					p_id = strtok(NULL, " ");
				}
				Q_free(buf);
			}
			dead = true;
		}
		if (dead)
			Cprintt("i died from impact\n",test);


		if (dead){
			//Com_Printf("Particle is dead and got deleted\n");
			if (current == draw_particles_list){
				draw_particles_list = current->next;
				temp = current;
				current = draw_particles_list;
			}else{
				last->next = current->next;
				temp = current;
				current = current->next;
				//last = current;
			}
			Q_free(temp);
			particles_count--;
		}else{
			last	= current;
			current	= current->next;
		}
	}


}

#define lhrandom(MIN,MAX) ((rand() & 32767) * (((MAX)-(MIN)) * (1.0f / 32767.0f)) + (MIN))
void PS_Add_Particle_To_Draw_List (ps_child_t *p){
	ps_plist_t	*pps_list,*ps_list,*pl;
	int i;
	float ft,v1,v2;
	//loat test;
	vec3_t	forward,right,up,test,test1;
	ps_list = draw_particles_list;
	

	pl = malloc(sizeof(ps_plist_t));
	memset(pl,0,sizeof(ps_plist_t));
	pl->particle = p;
	particles_count++;


	for (i=0;i<4;i++)
		p->color.color[i] = p->color.color_start[i];

	// do some randomizing
	//VectorNormalize(p->direction);
	if (p->random_direction.type == 1){
		for (i=0;i<3;i++)
			p->direction[i] = p->direction[i] + lhrandom(p->random_direction.min,p->random_direction.max);
	} else if (p->random_direction.type >= 2){
		v1 = lhrandom(p->random_direction.min,p->random_direction.max);
		v2 = lhrandom(p->random_direction.min,p->random_direction.max);
		PS_Rotate_Vector(p->direction,p->direction,v1,v2);
	}

	VectorScale(p->direction,p->speed.value,p->direction);
	

	if (p->random_origin.type){
		for (i=0;i<3;i++)
			p->origin[i] = p->origin[i] + lhrandom(p->random_origin.min,p->random_origin.max);
	}

	if (ps_list == NULL){
		draw_particles_list = pl;
		return;
	}
	if (p->light.unique_id)
		p->light.unique_id = unique_light_id++;
	while (ps_list){
		pps_list = ps_list;
		ps_list = ps_list->next;
	}
	pps_list->next = pl;
	return;

	

}

void PS_Spawn_Particles_From_Particle(char *identifier,ps_child_t *parent){
	ps_plist_t			*ps_rt;
	ps_child_t			*ps_found,*p;
	ps_rt = loaded_particles_list;
	if (ps_rt == NULL){
		Com_Printf("No Particle in the Particle List\n");
		return;
	}
	ps_found = NULL;
	//malloc(sizeof(ps_plist_t
	while (ps_rt != NULL){
		if (!strcmp(identifier,ps_rt->particle->identifier)){
			ps_found = ps_rt->particle;
			break;
		}
		ps_rt=ps_rt->next;
	}
	if (ps_found == NULL){
		Com_Printf("No Particle with the identifier \"%s\" in the Particlelist\n",identifier);
		return;
	}

	p = malloc(sizeof(ps_child_t));
	memcpy(p,ps_found,sizeof(ps_child_t));
	VectorCopy(parent->origin,p->origin);
	//VectorCopy(parent->direction,p->direction);
	VectorScale(parent->direction,p->speed.value,p->direction);
	p->starttime = r_refdef2.time;
	PS_Add_Particle_To_Draw_List(p);
}

void PS_Spawn_Particles_From_Parent(char *identifier,ps_parent_t *parent){
	ps_plist_t			*ps_rt;
	ps_child_t			*ps_found,*p;
	ps_rt = loaded_particles_list;


	ps_found = NULL;

	if (ps_rt == NULL){
		Com_Printf("No Particle in the Particle List\n");
		return;
	}
	//malloc(sizeof(ps_plist_t
	while (ps_rt != NULL){
		if (!strcmp(identifier,ps_rt->particle->identifier)){
			ps_found = ps_rt->particle;
			break;
		}
		ps_rt=ps_rt->next;
	}
	if (ps_found == NULL){
		Com_Printf("No Particle with the identifier \"%s\" in the Particlelist\n",identifier);
		return;
	}

	p = malloc(sizeof(ps_child_t));
	memcpy(p,ps_found,sizeof(ps_child_t));
	VectorCopy(parent->origin,p->origin);
	//VectorCopy(parent->direction,p->direction);
	VectorScale(parent->direction,p->speed.value,p->direction);
	p->starttime = r_refdef2.time;
	PS_Add_Particle_To_Draw_List(p);
	

}

void PS_Spawn_Parent(char *identifier){

	ps_parent_list_t	*ps_rt;
	ps_parent_t			*ps_found;
	ps_rt = ps_parent_list;
	
	if (ps_rt == NULL){
		Com_Printf("No Parent in the Parentlist\n");
		return;
	}
	while (ps_rt != NULL){
		if (!strcmp(identifier,ps_rt->parent->identifier)){
			ps_found = ps_rt->parent;
			break;
		}
		ps_rt=ps_rt->next;
	}
	if (ps_rt == NULL){
		Com_Printf("No Parent with the identifier \"%s\" in the Parentlist\n",identifier);
		return;
	}
	PS_Spawn_Particles_From_Parent(ps_found->plist,ps_found);
}

void PS_Spawn_Particle_From_Console (void){
	ps_parent_t p;
	vec3_t forward,right,up;
	VectorCopy(cl.simorg,p.origin);
	AngleVectors (cl.simangles,forward, right, up);
	VectorMA(p.origin,1,forward,p.direction);
	VectorCopy(forward,p.direction);
	//VectorScale(p.direction,p.speed,p.direction);

	PS_Spawn_Particles_From_Parent(Cmd_Argv(1),&p);
}

void PS_Add_Particle(void){


}


void PS_Add_Gravity_point (void) {
	VectorCopy(cl.simorg,gravity_point[atoi(Cmd_Argv(1))].origin);
	gravity_point[atoi(Cmd_Argv(1))].gravity = atof(Cmd_Argv(2));
	gravity_point[atoi(Cmd_Argv(1))].distance = atof(Cmd_Argv(3));
}


/*
	everything will be zero unless its set
	bounce		- b dies_by_bounce bounce_count bounce_modifier "spawns_on_bounce"
	hit			- h dies_by_hit "spawns_on_hit"
	time		- t dies_by_time "spawns_on_time"
	direction	- d x y z - added to parents d
	origin		- o x y z - added to parents o
	gravity		- g modifier
	speed		- s modifier dies_by_speed speed_limit


*/

void PS_Edit_Particle_Type (void){
	ps_child_t	*p;
	ps_plist_t	*pc;
	int i,hash;
	hash = Com_HashKey(Cmd_Argv(1));

	pc = loaded_particles_list;
	while (pc){
		if (pc->particle->hash == hash){
			break;
		}
	pc=pc->next;
	}
	if (pc == NULL){
		Com_Printf("No Particle with the identifier \"%s\" was found.\n",Cmd_Argv(1));
	}
	
	hash = Com_HashKey(Cmd_Argv(2));
	Com_Printf("%i\n",hash);
	p = pc->particle;
	i=3;
	switch (hash){
		case pt_bounce:
			p->bounce.dies_by = atoi(Cmd_Argv(i++));
			p->bounce.lmax = atoi(Cmd_Argv(i++));
			p->bounce.active = atoi(Cmd_Argv(i++));
			p->bounce.value = atof(Cmd_Argv(i++));
			if (!strcmp(Cmd_Argv(i),"NULL"))
				p->bounce.spawns = NULL;
			else
				p->bounce.spawns = Q_strdup(Cmd_Argv(i++));
			break;
		case pt_hit:
			p->impact.dies_by = atoi(Cmd_Argv(i++));
			if (!strcmp(Cmd_Argv(i),"NULL"))
				p->impact.spawns= NULL;
			else
				p->impact.spawns = Q_strdup(Cmd_Argv(i++));
			break;
		case pt_time:
			p->time.dies_by = atoi(Cmd_Argv(i++));
			p->time.value = atof(Cmd_Argv(i++));
			p->time.blends_by = atoi(Cmd_Argv(i++));
			break;
		case pt_direction:
			p->random_direction.type = atoi(Cmd_Argv(i++));
			p->random_direction.min = atof(Cmd_Argv(i++));
			p->random_direction.max = atof(Cmd_Argv(i++));

			//VectorSet(p->direction,atof(Cmd_Argv(i++)),atof(Cmd_Argv(i++)),atof(Cmd_Argv(i++)));
			break;
		case pt_origin:
			p->random_origin.type = atoi(Cmd_Argv(i++));
			p->random_origin.min = atof(Cmd_Argv(i++));
			p->random_origin.max = atof(Cmd_Argv(i++));
			//VectorSet(p->origin,atof(Cmd_Argv(i++)),atof(Cmd_Argv(i++)),atof(Cmd_Argv(i++)));
			break;
		case pt_gravity:
			p->g_modifier = atof(Cmd_Argv(i++));
			break;
		case pt_speed:
			p->speed.modifier = atof(Cmd_Argv(i++));
			p->speed.dies_by = atoi(Cmd_Argv(i++));
			p->speed.lmax = atof(Cmd_Argv(i++));
			p->speed.value = atof(Cmd_Argv(i++));
			if (!strcmp(Cmd_Argv(i),"NULL"))
				p->speed.spawns = NULL;
			else
				p->speed.spawns = Q_strdup(Cmd_Argv(i++));
			break;
		case pt_ival:
			p->interval.value = atof(Cmd_Argv(i++));
			if (!strcmp(Cmd_Argv(i),"NULL"))
				p->interval.spawns = NULL;
			else
				p->interval.spawns = Q_strdup(Cmd_Argv(i++));
			break;

		case pt_color:
			p->color.color_start[0] = atof(Cmd_Argv(i++));
			p->color.color_start[1] = atof(Cmd_Argv(i++));
			p->color.color_start[2] = atof(Cmd_Argv(i++));
			p->color.color_start[3] = atof(Cmd_Argv(i++));
			p->color.blends_by = atoi(Cmd_Argv(i++));
			p->color.color_end[0] = atof(Cmd_Argv(i++));
			p->color.color_end[1] = atof(Cmd_Argv(i++));
			p->color.color_end[2] = atof(Cmd_Argv(i++));
			p->color.color_end[3] = atof(Cmd_Argv(i++));
			break;

		case pt_sound:
			p->sound = S_PrecacheSound(Cmd_Argv(i));
			break;

		case pt_type:
			p->type = atoi(Cmd_Argv(i++));
			break;

		case pt_textures:
			p->texture = *GL_LoadPicImage(va("gfx/%s", Cmd_Argv(i)), Cmd_Argv(i), 0, 0, TEX_ALPHA | TEX_COMPLAIN);
			Com_Printf("%s %i\n",Cmd_Argv(i),p->texture);
			break;

		case pt_size:
			p->size = atof(Cmd_Argv(i++));
			p->growth = atof(Cmd_Argv(i++));
			break;

		case pt_light:
			p->light.unique_id = atoi(Cmd_Argv(i++));
			p->light.color = atoi(Cmd_Argv(i++));
			p->light.type = atoi(Cmd_Argv(i++));
			p->light.radius = atof(Cmd_Argv(i++));
			p->light.time = atof(Cmd_Argv(i++));
			p->light.custom= atoi(Cmd_Argv(i++));
			p->light.custom_color[0] = atof(Cmd_Argv(i++));
			p->light.custom_color[1] = atof(Cmd_Argv(i++));
			p->light.custom_color[2] = atof(Cmd_Argv(i++));
			break;
	}
	

}


void PS_Create_Particle_Type (void){
	ps_child_t	*p;
	ps_plist_t	*plist,*ppc,*pc;

	if (Cmd_Argc()<1){
		Com_Printf("We need an identifier\n");
		return;
	}
	p=malloc(sizeof(ps_child_t));
	memset (p, 0, sizeof(ps_child_t));
	plist=malloc(sizeof(ps_plist_t));
	memset (plist, 0, sizeof(ps_plist_t));
	p->identifier = Q_strdup(Cmd_Argv(1));
	p->hash = Com_HashKey(p->identifier);
	plist->particle = p;
	plist->next = NULL;
	if (loaded_particles_list == NULL){
		loaded_particles_list = plist;
		return;
	}

	pc = loaded_particles_list;
	while (pc){
		if (pc->particle->hash == p->hash){
			Com_Printf("A Particle with the identifier \"%s\" already exists.\n",p->identifier);
			free(p);
			free(plist);
			return;
		}
		ppc =pc;
		pc=pc->next;
	}

	ppc->next=plist;
	 
	return;
}

void PS_List_Particle_Types(void){
	ps_plist_t	*pslist;
	pslist = loaded_particles_list;
	while (pslist){
		Com_Printf("%s\n",pslist->particle->identifier);
		//Com_Printf("%i %i %i\n",(int)pslist->particle->color[0],(int)pslist->particle->color[1],(int)pslist->particle->color[2]);
		pslist=pslist->next;
	}
	return;
}

// 
void PS_Rotate_Vector (vec3_t dst,vec3_t org,float degree_x, float degree_y){
	vec3_t	forward,right,up;
	float length;

	AngleVectors(org,forward,right,up);
	length = forward[0] * forward[0] + forward[1] * forward[1] + forward[2] * forward[2];

	dst[0] = length * cos(DEG2RAD(-org[0] + degree_x)) * cos(DEG2RAD(org[1] + degree_y));
	dst[1] = length * cos(DEG2RAD(-org[0] + degree_x)) * sin(DEG2RAD(org[1] + degree_y));
	dst[2] = length * sin(DEG2RAD(-org[0] + degree_x));

}


void PS_Pcount (void){

	Com_Printf("%i\n",particles_count);

}

void PS_Init (void){
	Cmd_AddCommand("ps_list_particle_types",PS_List_Particle_Types);
	Cmd_AddCommand("ps_create_particle_type",PS_Create_Particle_Type);
	Cmd_AddCommand("ps_add",PS_Add_Particle);
	Cmd_AddCommand("ps_gpoint",PS_Add_Gravity_point);
	Cmd_AddCommand("ps_edit_particle_type",PS_Edit_Particle_Type);
	Cmd_AddCommand("ps_spawn_particle_from_console",PS_Spawn_Particle_From_Console);
	Cmd_AddCommand("ps_pcount",PS_Pcount);
}







