#include "python/Python.h"
#include "quakedef.h"
#include "gl_texture.h"

extern qbool disable_culling;

PyObject		*dict_var;
PyObject		*dict_stats_self;

typedef struct {
	int			texnum;
	char		identifier[MAX_QPATH];
	char		*pathname;
	int			width, height;
	int			scaled_width, scaled_height;
	int			texmode;
	unsigned	crc;
	int			bpp;
} gltexture_t;

typedef struct p_light_s {
	int				unique_id;
	int				color;
	int				type;
	float			radius;
	float			time;
	qbool			custom;
	float			custom_color[3];
	
} p_light_t;



typedef struct p_color_s {
	int		blends_by;
	float	color1[4];
	float	color2[4];
	float	color1_s[4];
	float	color1_e[4];
	float	color2_s[4];
	float	color2_e[4];
} p_color_t;

typedef struct p_random_s {
	int		type;
	float	value1;
	float	value2;
	float	value3;
	float	value4;
	float	value5;
	float	value6;
	float	value7;
	float	value8;
} p_random_t;

typedef struct p_info_s {
	qbool	active;
	qbool	dies_by;
	int		blends_by;
	char	*spawns;
	float	value1;
	float	value2;
	float	value3;
	float	value4;
	float	value5;
	float	value6;
	float	value7;
	float	value8;
} p_info_t;


typedef struct ps_child_s {
	char			*identifier;					// name
	int				hash;							// like the cvars :>
	int				type;							// billboard etc etc
	float			starttime;						// time we spawned
	p_info_t		time;
	p_info_t		interval;
	p_info_t		bounce;
	p_light_t		light;
	p_info_t		speed;
	p_info_t		impact;
	p_color_t		color;
	p_random_t		random_direction;
	p_random_t		random_origin;
	p_random_t		rotation;
	p_random_t		g_point;

	//qbool			dies_by_impact;					// we do expire when whe hit something
	qbool			g_affected;						// are we affected by a gravity point
	//qbool			done;							// er ?
	float			g_modifier;						// gravity modifier
	char			*on_death_spawn;				// list of particles spawned when particle dies
	//char			*on_impact_spawn;				// list of particles spawned when particle impact

	vec3_t			origin;							// origin
	vec3_t			endorg;							//
	vec3_t			normal;							// the normal vector if we hit smth nice one for decals :D
	vec3_t			old_origin[9];
	vec3_t			direction;						// direction

	char			*python_module;
	char			*python_function;

	

	sfx_t			*sound;
	mpic_t			texture;
	qbool			sound_played;
	float			size;
	float			growth;
	unsigned int				unique_id;
	int				spawned_by;
	int				hit;
	int				bounced;
	//int				blend_over_time;				// 1 = start at 0 end at 255

} ps_child_t;

extern ps_child_t *python_particle;

typedef struct PyFunctions_s{
	PyObject		*function;
	struct PyFunctions_s	*next;
} PyFunctions_t;


void Draw_3D_String(vec3_t	location,	vec3_t	direction, char *string,int x,int y);



static PyObject *Python_Create_Variable(PyObject *self,PyObject *args){
	char *s;
	char *s1;
	cvar_t	*var;
	char	st[128];
	char	st1[128];

	if (! PyArg_ParseTuple(args, "ss",&s,&s1)) {
		Py_RETURN_NONE;
	}

	
	var = Q_malloc(sizeof(cvar_t));
	memset(var,0,sizeof(cvar_t));
	strcpy(st,s);
	strcpy(st1,s1);
	var->name = &st;
	var->defaultvalue = &st1;
	

	Cvar_Register(var);
	Py_RETURN_NONE;

	}

static PyObject *Python_Draw_3D_String(PyObject *self,PyObject *args){
	vec3_t	loc,dir;
	int x,y;
	char *s;
	
	if (! PyArg_ParseTuple(args, "ffffffsii",&loc[0],&loc[1],&loc[2],&dir[0],&dir[1],&dir[2],&s,&x,&y)) {
		Py_RETURN_NONE;
	}
	
	Draw_3D_String(loc,dir,s,x,y);
	Py_RETURN_NONE;
}

void Draw_3D_String_Colored(vec3_t	location,	vec3_t	direction, char *string,int x,int y, float r, float g, float b);

static PyObject *Python_Draw_3D_String_Colored(PyObject *self,PyObject *args){
	vec3_t	loc,dir;
	int x,y;
	float r,g,b;
	char *s;
	
	if (! PyArg_ParseTuple(args, "ffffffsiifff",&loc[0],&loc[1],&loc[2],&dir[0],&dir[1],&dir[2],&s,&x,&y,&r,&g,&b)) {
		Py_RETURN_NONE;
	}
	
	Draw_3D_String_Colored(loc,dir,s,x,y,r,g,b);
	Py_RETURN_NONE;
}

void Draw_3D_Pic (vec3_t	location, vec3_t	direction, int gl, float x, float y, float width, float height, float r, float g, float b, float a, float scale_x, float scale_y);
static PyObject *Python_Draw_3D_Pic(PyObject *self,PyObject *args){
	vec3_t	loc,dir;
	float r,g,b,a,sx,sy,x,y,w,h;
	char *s;
	gltexture_t	*pic = NULL;
	
	if (! PyArg_ParseTuple(args, "ffffffsffffffffff",&loc[0],&loc[1],&loc[2],&dir[0],&dir[1],&dir[2],&s,&x,&y,&w,&h,&r,&g,&b,&a,&sx,&sy)) {
		Py_RETURN_NONE;
	}
	pic = GL_FindTexture(s);
	if (!pic)
		Py_RETURN_NONE;

	Draw_3D_Pic(loc,dir,pic->texnum,x,y,w,h,r,g,b,a,sx,sy);
	Py_RETURN_NONE;
}


void Draw_3D_RectangleRGB (vec3_t	location, vec3_t	direction, int x, int y, int w, int h, float r, float g, float b, float a, float f,float t);
static PyObject *Python_Draw_3D_Rectangle(PyObject *self,PyObject *args){
	vec3_t	loc,dir;
	int x,y,w,h;
	float r,g,b,f,a,t;
	
	if (! PyArg_ParseTuple(args, "ffffffiiiiffffff",&loc[0],&loc[1],&loc[2],&dir[0],&dir[1],&dir[2],&x,&y,&w,&h,&r,&g,&b,&a,&f,&t)) {
		Py_RETURN_NONE;
	}
	Draw_3D_RectangleRGB(loc,dir,x,y,w,h,r,g,b,a,f,t);
	Py_RETURN_NONE;
}


static PyObject *Python_Com_Printf(PyObject *self,PyObject *args){
	char *s;
	
	if (! PyArg_ParseTuple(args, "s", &s)) {
		Py_RETURN_NONE;
	}
	Com_Printf("%s\n",s);
	
	Py_RETURN_NONE;
}

static PyObject *Python_Cbuf_AddText(PyObject *self,PyObject *args){
	char *s;
	
	if (! PyArg_ParseTuple(args, "s", &s)) {
		Py_RETURN_NONE;
	}
	Cbuf_AddText(s);
	Py_RETURN_NONE;
}

static PyObject *Python_Setup_Matrix(PyObject *self,PyObject *args){
	vec3_t	loc,dir;
	float f = 0;
	char *s;
	
	if (! PyArg_ParseTuple(args, "fffffff",&loc[0] ,&loc[1] ,&loc[2] ,&dir[0] ,&dir[1] ,&dir[2] ,&f)) {
		Py_RETURN_NONE;
	}
	glPushMatrix();
	if (loc[0] || loc[1] || loc[2])
		glTranslatef(loc[0],loc[1],loc[2]);

	if (dir[0] || dir[1] || dir[2]){
		glRotatef (dir[1], 0, 0, 1);
		glRotatef (dir[0], 0, 1, 0);
		glRotatef (-dir[2], 1, 0, 0);
		}
	if (f)
		glScalef(f,f,f);

	Py_RETURN_NONE;
}

static PyObject *Python_Pop_Matrix(PyObject *self,PyObject *args){
	char *s;
	
	//if (! PyArg_ParseTuple(args, "s", &s)) {
	//	Py_RETURN_NONE;
	//}
	glPopMatrix();
	Py_RETURN_NONE;
}

model_t *Mod_FindName (char *name);
void R_DrawAliasModel (entity_t *ent);

static PyObject *Python_Draw_Model(PyObject *self,PyObject *args){
	char *s;
	entity_t ent;
	vec3_t	loc,dir;

	model_t	*mod = NULL;
	
	
	if (! PyArg_ParseTuple(args, "ffffffs",&loc[0] ,&loc[1] ,&loc[2] ,&dir[0] ,&dir[1] ,&dir[2] ,&s)) {
		Py_RETURN_NONE;
	}

	//ent = Q_malloc(sizeof(entity_t));
	memset(&ent,0,sizeof(entity_t));

	mod = Mod_FindName(s);
	ent.model = Mod_FindName(s);//mod;

	
	
	if (ent.model == NULL)
		Py_RETURN_NONE;
	VectorSet(ent.origin,loc[0],loc[1],loc[2]);
	VectorSet(ent.angles,dir[0],dir[1],dir[2]);
	currententity = &ent;
	glEnable(GL_DEPTH_TEST);
	switch (ent.model->type){
		case mod_alias:
					disable_culling = 1;
					R_DrawAliasModel(&ent);
					disable_culling = 0;
				break;
		case mod_alias3:
				R_DrawAlias3Model (&ent);
				break;
		case mod_brush:
				R_DrawBrushModel (&ent);
				break;
		case mod_sprite:
				R_DrawSpriteModel (&ent);
				break;
		}
	


	glDisable(GL_DEPTH_TEST);
	glPushMatrix();
	
	glPopMatrix();
	//free(ent);
	Py_RETURN_NONE;
}

static PyMethodDef Console_methods [] = {
	{"Com_Printf",(PyCFunction)Python_Com_Printf,METH_VARARGS,"Calls Com_Printf from python\n"},
	{"Cbuf_AddText",(PyCFunction)Python_Cbuf_AddText,METH_VARARGS,"Calls Cbuf_AddText from python\n"},
	{"ThreeDee_String",(PyCFunction)Python_Draw_3D_String,METH_VARARGS,"Draws 3d text anywhere !\n"},
	{"Create_Variable",(PyCFunction)Python_Create_Variable,METH_VARARGS,"Creates a variable !\n"},
	{"ThreeDee_String_Colored",(PyCFunction)Python_Draw_3D_String_Colored,METH_VARARGS,"Draws colored 3d text anywhere !\n"},
	{"ThreeDee_Rectangle",(PyCFunction)Python_Draw_3D_Rectangle,METH_VARARGS,"Draws colored rectangle !\n"},
	{"ThreeDee_Picture",(PyCFunction)Python_Draw_3D_Pic,METH_VARARGS,"Draws Picture !\n"},
	{"ThreeDee_Setup_Matrix",(PyCFunction)Python_Setup_Matrix,METH_VARARGS,"Sets up glmatrix !\n"},
	{"ThreeDee_Pop_Matrix",(PyCFunction)Python_Pop_Matrix,METH_VARARGS,"Pops Matrix !\n"},
	{"ThreeDee_Draw_Model",(PyCFunction)Python_Draw_Model,METH_VARARGS,"Draws a model !\n"},
	{NULL,NULL,0,NULL}
	
};
//PyDict_SetItemString(
void Python_Call(char *module, char *function, PyObject *arguments);
void Python_Call_Particle(char *module,char *function,ps_child_t *p){
	PyObject *pArgs,*pValue,*dict;
	int i = 0;
	int x;
	//long t = 200;

	pArgs = PyTuple_New(38);

	if (pArgs == NULL)
		Com_Printf("ERROR !\n");

	

	//p->unique_id;



	//return;
	// refresh the particle info in the python interpreter
	pValue = Py_BuildValue("i",p->unique_id);
	PyTuple_SetItem(pArgs, i++, pValue);
	pValue = Py_BuildValue("i",p->spawned_by);
	PyTuple_SetItem(pArgs, i++, pValue);
	pValue = Py_BuildValue("i",p->time.dies_by);
	PyTuple_SetItem(pArgs, i++, pValue);
	pValue = Py_BuildValue("f",p->time.value1);
	PyTuple_SetItem(pArgs, i++, pValue);
	pValue = Py_BuildValue("f",p->time.value2);
	PyTuple_SetItem(pArgs, i++, pValue);
	pValue = Py_BuildValue("f",p->time.value6);
	PyTuple_SetItem(pArgs, i++, pValue);
	pValue = Py_BuildValue("i",p->color.blends_by);
	PyTuple_SetItem(pArgs, i++, pValue);
	//7
	for (x=0;x<4;x++){
		pValue = Py_BuildValue("f",p->color.color1[x]);
		PyTuple_SetItem(pArgs, i++, pValue);
	}
	//11
	for (x=0;x<4;x++){
		pValue = Py_BuildValue("f",p->color.color2[x]);
		PyTuple_SetItem(pArgs, i++, pValue);
	}
	//15
	for (x=0;x<4;x++){
		pValue = Py_BuildValue("f",p->color.color1_s[x]);
		PyTuple_SetItem(pArgs, i++, pValue);
	}
	//19
	for (x=0;x<4;x++){
		pValue = Py_BuildValue("f",p->color.color1_e[x]);
		PyTuple_SetItem(pArgs, i++, pValue);
	}
	//23
	for (x=0;x<4;x++){
		pValue = Py_BuildValue("f",p->color.color2_s[x]);
		PyTuple_SetItem(pArgs, i++, pValue);
	}
	//27
	for (x=0;x<4;x++){
		pValue = Py_BuildValue("f",p->color.color2_e[x]);
		PyTuple_SetItem(pArgs, i++, pValue);
	}
	//31

	for (x=0;x<3;x++){
		pValue = Py_BuildValue("f",p->direction[x]);
		PyTuple_SetItem(pArgs, i++, pValue);
	}
	//34

	for (x=0;x<3;x++){
		pValue = Py_BuildValue("f",p->origin[x]);
		PyTuple_SetItem(pArgs, i++, pValue);
	}
	//37

	pValue = Py_BuildValue("i",p->hit);
	PyTuple_SetItem(pArgs, i++, pValue);
	//38
	
	
	//pValue = Py_BuildValue("f",125);
	//pValue = PyInt_FromLong(x);
	//pValue = Py_BuildValue("s","string");
	//Com_Printf("%i\n",PyTuple_SetItem(pArgs, i++, pValue));
/*	pValue = Py_BuildValue("f",p->time.value2);
	PyTuple_SetItem(pArgs, i++, pValue);
	//pValue = Py_BuildValue("f",p->time.value2);
	//PyTuple_SetItem(pArgs, i++, pValue)
*/
	Python_Call(module,"setup_particle",pArgs);
	
	pArgs = PyTuple_New(1);

	// calls the actual function
	pValue = Py_BuildValue("i",p->unique_id);
	PyTuple_SetItem(pArgs, 0, pValue);
	Python_Call(module,function,pArgs);
	//Com_Printf("Test\n");
	
}

void Python_Callback_Color(int blends,	float *c1);

static PyObject *Python_Particle_Color(PyObject *self,PyObject *args){
	float		c1[4];
	int		blends;

	PyErr_Clear();
	if (!PyArg_ParseTuple(args, "iffff",&blends,&c1[0],&c1[1],&c1[2],&c1[3])){
		Py_RETURN_NONE;
	}

	Python_Callback_Color(blends,&c1[0]);
	Py_RETURN_NONE;
}

static PyObject *Python_Particle_Text(PyObject *self,PyObject *args){
	Com_Printf("Test !\n");
	Py_RETURN_NONE;
}

void Python_Callback_Spawn(char *s);
static PyObject *Python_Particle_Spawn(PyObject *self,PyObject *args){
	char *s;


	if (! PyArg_ParseTuple(args, "s", &s)) {
		Py_RETURN_NONE;
	}

	Python_Callback_Spawn(s);

	
	Py_RETURN_NONE;
}

void Python_Callback_Type(int type);

static PyObject *Python_Particle_Type(PyObject *self,PyObject *args){
	int i;

	if (! PyArg_ParseTuple(args, "i", &i)) {
		Py_RETURN_NONE;
	}
	Python_Callback_Type(i);
	Py_RETURN_NONE;
}

static PyObject *Python_Particle_Direction(PyObject *self,PyObject *args){
	float		c1[3];
	int			i;

	PyErr_Clear();
	if (!PyArg_ParseTuple(args, "fff",&c1[0],&c1[1],&c1[2])){
		Py_RETURN_NONE;
	}

	for (i=0;i<3;i++)
		python_particle->direction[i] = c1[i];
	Py_RETURN_NONE;
}

static PyObject *Python_Particle_Origin(PyObject *self,PyObject *args){
	float		c1[3];
	int			i;

	PyErr_Clear();
	if (!PyArg_ParseTuple(args, "fff",&c1[0],&c1[1],&c1[2])){
		Py_RETURN_NONE;
	}

	for (i=0;i<3;i++)
		python_particle->origin[i] = c1[i];
	Py_RETURN_NONE;
}


static PyMethodDef Particle_methods [] = {
	{"Color",(PyCFunction)Python_Particle_Color,METH_VARARGS,"Sets Color1 of the current particle\n"},
	{"Text",(PyCFunction)Python_Particle_Text,METH_VARARGS,"Sets Color of the current particle1\n"},
	{"Spawn",(PyCFunction)Python_Particle_Spawn,METH_VARARGS,"Spawns a particle from python\n"},
	{"Type",(PyCFunction)Python_Particle_Type,METH_VARARGS,"Changes the type of a particle\n"},
	{"Direction",(PyCFunction)Python_Particle_Direction,METH_VARARGS,"Changes the direction vector of a particle\n"},
	{"Origin",(PyCFunction)Python_Particle_Origin,METH_VARARGS,"Changes the Origin vector of a particle\n"},
	{NULL,NULL,0,NULL}
};


	




qbool python_initialized;
PyFunctions_t *Python_Functions = NULL;

void Python_Hooks(void){
	
	
	Py_InitModule("console",Console_methods);
	Py_InitModule("qwparticle",Particle_methods);

}

void Python_Load(char *file){
		PyObject *pName,*pModule,*pFunc;
		PyFunctions_t *temp;


		if (!python_initialized)
			return;

		pName = PyString_FromString(file);

		pModule = PyImport_Import(pName);
		Py_DECREF(pName);

		if (pModule == NULL){
			Com_Printf("Python - Loading %s failed\n", file);
			return;
		}else{
			Com_Printf("Python - Loaded %s %i\n", PyModule_GetName(pModule),pModule);
		}

		if(Python_Functions == NULL){
			Python_Functions = (PyFunctions_t*) malloc(sizeof(PyFunctions_t));
			memset(Python_Functions,0,sizeof(PyFunctions_t));
			Python_Functions->function = pModule;
		}else{
			temp = Python_Functions;
			while (temp->next)
				temp=temp->next;
			temp->next = (PyFunctions_t*) malloc(sizeof(PyFunctions_t));
			memset(temp->next,0,sizeof(PyFunctions_t));
			temp->next->function = pModule;

		}


		
		PyObject_SetAttrString(pModule,(char *)"variables", dict_var);
		PyObject_SetAttrString(pModule,(char *)"stats", dict_stats_self);
		
		pFunc = PyObject_GetAttrString(pModule,"Init");


		if (pFunc == NULL){
			Com_Printf("Python - Loading %s failed it has no Init function\n", file);
			return;
		}else{
			Com_Printf("Python - Functions in %s loaded\n", file);
		}
		PyObject_CallObject(pFunc,NULL);
		Py_DECREF(pFunc);

	}

	PyFunctions_t *Python_GetObjectByName(char *name){
		PyFunctions_t *temp;
		int i= 0;

		temp = Python_Functions;

		while (temp){
			//Com_Printf("%s %i",PyModule_GetName(temp->function),i++);
			if (!strcmp(PyModule_GetName(temp->function),name))
				return temp;
			temp=temp->next;
		}
		return NULL;
		
	}
void Python_Init(void);
void Python_Quit(void);
void Python_Reload(char *module){

		Python_Quit();
		Python_Init();

	}

void Python_Call(char *module, char *func_name, PyObject *arguments){
	PyObject *mod ,*func ;
	PyFunctions_t	*t;

	t = Python_GetObjectByName(module);
	
	if (t == NULL){
		//Com_Printf("Python -CALL - No module with the name %s\n",module);
		return;
	}
	mod = t->function;
	func = PyObject_GetAttrString(mod,func_name);
	
	if (func == NULL){
		Com_Printf("Python - Calling %s in module %s failed\n", func_name,module);
		return;
	}else{
		//Com_Printf("Python - Calling %s in module %s success !\n", function,module);
	}
	PyObject_CallObject(func,arguments);
	Py_DECREF(func);
	
}


	void Particle_Reload_From_Console(void){
		Python_Reload(Cmd_Argv(1));
	}

	void Python_Call_From_Console (void){
			
		Python_Call(Cmd_Argv(1),Cmd_Argv(2),NULL);

		}

void	Python_Reinit_From_Console(void){
		//Py_Finalize();
		python_initialized = 0;
		Python_Init();


}

void Python_Init(void){
	char newpath[1024];
	PyObject		*dict1,*temp1,*temp2,*temp3;
	PyFunctions_t	*mod;
	

	Py_Initialize();


	if (!python_initialized){
		strcpy(newpath,Py_GetPath());
		strcat(newpath,"\\python");
		PySys_SetPath(newpath);
		Cmd_AddCommand("python_reload_module",Particle_Reload_From_Console);
		Cmd_AddCommand("python_call",Python_Call_From_Console);
		Cmd_AddCommand("python_reinit",Python_Reinit_From_Console);
	}
	//Com_Printf("%s\n%s\n",Py_GetPath(),newpath);
	

	if (Py_IsInitialized()){
		Com_Printf("Python - Initialized\n");
		Com_Printf("Python - Version %s\n",Py_GetVersion());
		dict_var = PyDict_New();
		PyDict_SetItemString(dict_var, "variables", PyEval_GetBuiltins(  ));
		dict_stats_self = PyDict_New();
		PyDict_SetItemString(dict_var, "stats", PyEval_GetBuiltins(  ));
		python_initialized = true;
		Python_Hooks();
	}else{
		Com_Printf("Python - Initialization failed\n");
		python_initialized = false;
		return;
	}

	Python_Load("particles");

	Python_Load("main");
	
	
	

}

void Python_Quit(void){
	PyFunctions_t *temp,*old;

	temp = Python_Functions;
	while (temp){
		Py_DECREF(temp->function);
		old = temp;
		temp = temp->next;
		free(old);
		
	}
	Python_Functions = NULL;
	Py_Finalize();
}

void Python_Frame(void){
		PyObject *pArgs,*pValue;
		int i = 0;
		pArgs = PyTuple_New(1);
		pValue = Py_BuildValue("d",Sys_DoubleTime());
		PyTuple_SetItem(pArgs, i++, pValue);
		Python_Call("main","frame",pArgs);
		//Com_Printf("%f\n",PyDict_Size(dict_var));

}

void HUD_3D_Setup(void);
void HUD_3D_End(void);

void Python_3D_Frame(void){
		PyObject *pArgs,*pValue;
		int i = 0;
		player_info_t	*s;
		
		if (!cl.worldmodel)
			return;

		s = &cl.players[cl.playernum];
		PyDict_SetItemString(dict_stats_self, "name", PyString_FromString(s->name));
		PyDict_SetItemString(dict_stats_self, "health", PyInt_FromLong(cl.stats[STAT_HEALTH]));
		PyDict_SetItemString(dict_stats_self, "armor", PyInt_FromLong(cl.stats[STAT_ARMOR]));
		PyDict_SetItemString(dict_stats_self, "shells", PyInt_FromLong(cl.stats[STAT_SHELLS]));
		PyDict_SetItemString(dict_stats_self, "nails", PyInt_FromLong(cl.stats[STAT_NAILS]));
		PyDict_SetItemString(dict_stats_self, "rockets", PyInt_FromLong(cl.stats[STAT_ROCKETS]));
		PyDict_SetItemString(dict_stats_self, "cells", PyInt_FromLong(cl.stats[STAT_CELLS]));
		PyDict_SetItemString(dict_stats_self, "active_weapon", PyInt_FromLong(cl.stats[STAT_ACTIVEWEAPON]));
		PyDict_SetItemString(dict_stats_self, "items", PyInt_FromLong(cl.stats[STAT_ITEMS]));

 


		HUD_3D_Setup();

		pArgs = PyTuple_New(1);
		pValue = Py_BuildValue("d",Sys_DoubleTime());
		PyTuple_SetItem(pArgs, i++, pValue);
		Python_Call("main","threedee_frame",pArgs);			
		if (PyErr_Occurred())
			PyErr_Print();

		HUD_3D_End();

	}


void Python_RegisterVariable(cvar_t *var){
	if (!python_initialized)
		return;
		PyDict_SetItemString(dict_var, var->name, PyString_FromString(var->string));


	}






